#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    Utils
    =====
     Utils manage function needed by converter
"""

import os
import glob
import re
import time
import ntpath
from collections import OrderedDict

import colorama


def init_colorama():
    if 'PYCHARM_HOSTED' in os.environ:
        convert = False  # in PyCharm, we should disable convert
        strip = False
        print("Hi! You are using PyCharm")
    else:
        convert = None
        strip = None
    colorama.init(convert=convert, strip=strip)


init_colorama()

DONE = colorama.Fore.GREEN + colorama.Style.BRIGHT
OK = colorama.Fore.CYAN + colorama.Style.BRIGHT
WARN = colorama.Fore.YELLOW + colorama.Style.BRIGHT
FAIL = colorama.Fore.RED + colorama.Style.BRIGHT
ENDC = colorama.Fore.RESET + colorama.Style.RESET_ALL


class Utils:

    @staticmethod
    def lists_of_settings_to_merge():
        return [
            'defines',
            'inc_dirs',
            'target_link_dirs',
            'PDB_OUTPUT_DIRECTORY',
            'PDB_NAME'
        ]

    def init_context_current_setting(self, context):
        """
        Define settings of converter.

        :param context: converter context
        :type context: Context
        """

        conf = context.current_setting[0]
        arch = context.current_setting[1]

        setting = (conf, arch)
        context.settings[setting] = {
            'conf': conf,
            'arch': arch,
            'TARGET_NAME': [],
            'target_type': '',
            'OUTPUT_DIRECTORY': [],
            'inc_dirs_list': [],
            'add_lib_deps': [],
            'ARCHIVE_OUTPUT_DIRECTORY': [],
            'ARCHIVE_OUTPUT_NAME': [],
            'VS_GLOBAL_KEYWORD': [],
            'pre_build_events': [],
            'pre_link_events': [],
            'post_build_events': [],
        }


def take_name_from_list_case_ignore(context, search_list, name_to_search):
    """
    Return real name of name to search

    :param context: the context of converter
    :type context: Context
    :param search_list: list to makje research
    :type search_list: list
    :param name_to_search: name tosearch in list
    :type name_to_search: str
    :return: real name
    :rtype: str
    """

    real_name = ''
    for item in search_list:
        if item.lower() == name_to_search.lower():
            real_name = item
            break

    if real_name == '':
        message(context, '{0} file is absent at filesystem. Ignoring but check it!!'
                .format(name_to_search), 'warn')
        return ''

    search_list.remove(real_name)
    return real_name


def is_settings_has_data(sln_configurations_map, settings, settings_key, arch=None, conf=None):
    if arch:
        arch = get_mapped_arch(sln_configurations_map, arch)

    for sln_setting in sln_configurations_map:
        mapped_setting_name = sln_configurations_map[sln_setting]
        if mapped_setting_name not in settings:
            continue
        mapped_setting = settings[mapped_setting_name]
        if settings_key in mapped_setting:
            if arch and mapped_setting['arch'] != arch:
                continue
            if conf and mapped_setting['conf'] != conf:
                continue
            if mapped_setting[settings_key]:
                return True
    return False


def write_property_of_setting_f(cmake_file,
                                property_indent,
                                config_condition_expr,
                                property_value,
                                width,
                                **kwargs):
    del width

    separator = kwargs['separator']
    quotes = '"' if kwargs['in_quotes'] else ''

    prop_value_indent = config_indent = property_indent + '    '
    config_condition_expr_str = ''
    if config_condition_expr is not None:
        config_condition_expr_str = '$<' + config_condition_expr + ':'

    property_value_str = get_str_value_from_property_value(property_value, separator)
    property_list_str = property_value_str.split('\n')

    if config_condition_expr:
        cmake_file.write(
            '{0}{1}{2}\n'.format(config_indent, quotes + config_condition_expr_str, quotes)
        )
        prop_value_indent = config_indent + '    '

    for prop in property_list_str:
        cmake_file.write(
            '{0}{1}\n'.format(prop_value_indent, quotes + prop + quotes)
        )

    if config_condition_expr:
        cmake_file.write('{0}{1}\n'.format(config_indent, quotes + '>' + quotes))


def get_str_value_from_property_value(property_value, separator):
    if isinstance(property_value, list):
        return separator.join(property_value)

    raise str('property value must be a list')


def get_mapped_arch(sln_setting_2_project_setting, arch):
    for setting in sln_setting_2_project_setting:
        if setting[1] == arch:
            return sln_setting_2_project_setting[setting][1]
    return None


def write_selected_sln_setting(cmake_file,
                               settings,
                               sln_setting_2_project_setting,
                               sln_setting,
                               has_property_value,
                               command_indent,
                               sln_conf,
                               config_expressions,
                               max_config_condition_width,
                               **kwargs
                               ):

    begin_text = kwargs['begin_text']
    property_name = kwargs['property_name']
    indent = kwargs['indent']
    write_setting_property_func = kwargs['write_setting_property_func']

    mapped_setting = settings[sln_setting_2_project_setting[sln_setting]]
    if property_name in mapped_setting:
        if mapped_setting[property_name]:
            if not has_property_value:
                begin_text = begin_text.replace('\n', '\n' + indent + command_indent)
                if begin_text:
                    cmake_file.write('{0}{1}\n'.format(indent + command_indent, begin_text))
                has_property_value = True

            config_condition_expr = None
            if sln_conf is not None:
                config_condition_expr = '$<CONFIG:{0}>'.format(sln_conf)
                config_expressions.append(config_condition_expr)
            write_setting_property_func(cmake_file,
                                        indent + command_indent,
                                        config_condition_expr,
                                        mapped_setting[property_name],
                                        max_config_condition_width,
                                        **kwargs
                                        )
    return has_property_value


def write_footer_of_settings(cmake_file,
                             command_indent,
                             config_expressions,
                             has_property_value,
                             **kwargs):

    default = kwargs['default']
    end_text = kwargs['end_text']
    indent = kwargs['indent']

    if has_property_value:
        if default:
            cmake_file.write('{0}    $<$<NOT:$<OR:{1}>>:{2}>\n'
                             .format(indent + command_indent, ','.join(config_expressions),
                                     default))
        end_text = end_text.replace('\n', '\n' + indent + command_indent)
        if end_text:
            cmake_file.write('{0}{1}\n'.format(indent + command_indent, end_text))


# pylint: disable=R0914
# pylint: disable=R0913


def write_property_of_settings(cmake_file, settings, sln_setting_2_project_setting, **kwargs):
    """
    Write property of given settings.

    :param cmake_file: CMakeLists.txt IO wrapper
    :type cmake_file: _io.TextIOWrapper
    :param settings: global settings
    :type settings: dict
    :param sln_setting_2_project_setting: solution settings attached to project
    :type sln_setting_2_project_setting: dict
    :param kwargs: begin of text

    kwargs:
    indent: indent to use when writing
    indent: str
    default: default text to add
    default: None | str
    separator: separator for property list
    separator: ; | str
    in_quotes: Enclose configuration settings in quotes
    in_quotes: False | bool
    write_setting_property_func: function for writing property for setting
    write_setting_property_func: write_property_of_setting | lambda

    """

    property_name = kwargs['property_name']

    # defaults
    if 'separator' not in kwargs:
        kwargs['separator'] = ';'
    if 'indent' not in kwargs:
        kwargs['indent'] = ''
    if 'default' not in kwargs:
        kwargs['default'] = None
    if 'in_quotes' not in kwargs:
        kwargs['in_quotes'] = False
    if 'write_setting_property_func' not in kwargs:
        kwargs['write_setting_property_func'] = write_property_of_setting_f

    indent = kwargs['indent']

    has_property_value = False

    command_indent = ''
    config_expressions = []
    has_property_value = write_selected_sln_setting(
        cmake_file, settings, sln_setting_2_project_setting, (None, None),
        has_property_value,
        command_indent,
        None,
        config_expressions,
        0,
        **kwargs
    )

    write_footer_of_settings(cmake_file,
                             command_indent,
                             config_expressions,
                             has_property_value,
                             **kwargs)

    max_config_condition_width = 0
    settings_of_arch = OrderedDict()
    for sln_setting in sln_setting_2_project_setting:
        arch = sln_setting[1]
        if arch is None:
            continue
        conf = sln_setting[0]
        if conf is not None:
            length = len('$<CONFIG:{0}>'.format(conf))
            if length > max_config_condition_width:
                max_config_condition_width = length
        if arch not in settings_of_arch:
            settings_of_arch[arch] = OrderedDict()
        settings_of_arch[arch][sln_setting] = sln_setting

    single_arch = len(settings_of_arch) == 1
    command_indent = ''
    first_arch = True
    for arch in settings_of_arch:
        has_data = is_settings_has_data(sln_setting_2_project_setting, settings, property_name,
                                        arch)
        if not has_data:
            continue

        if not single_arch:
            command_indent = '    '
            if first_arch:
                cmake_file.write('{0}if(\"${{CMAKE_VS_PLATFORM_NAME}}\" STREQUAL \"{1}\")\n'
                                 .format(indent, arch))
            else:
                cmake_file.write('{0}elseif(\"${{CMAKE_VS_PLATFORM_NAME}}\" STREQUAL \"{1}\")\n'
                                 .format(indent, arch))
        first_arch = False
        has_property_value = False
        config_expressions = []
        for sln_setting in settings_of_arch[arch]:
            sln_conf = sln_setting[0]
            if sln_setting_2_project_setting[sln_setting] not in settings:
                continue

            has_property_value = write_selected_sln_setting(
                cmake_file, settings, sln_setting_2_project_setting, sln_setting,
                has_property_value,
                command_indent,
                sln_conf,
                config_expressions,
                max_config_condition_width,
                **kwargs
            )
        write_footer_of_settings(cmake_file,
                                 command_indent,
                                 config_expressions,
                                 has_property_value,
                                 **kwargs)
    if not first_arch and not single_arch:
        cmake_file.write('{0}endif()\n'.format(indent))

    return not first_arch

# pylint: enable=R0914
# pylint: enable=R0913


def get_global_project_name_from_vcxproj_file(vcxproj):
    """
    Return global project name from ".vcxproj" file

    :param vcxproj: vcxproj data
    :type vcxproj: dict
    :return: project name
    :rtype: str
    """

    project_name_node = vcxproj['tree'].xpath('//ns:ProjectName', namespaces=vcxproj['ns'])
    project_name = None
    if project_name_node:
        project_name_value = project_name_node[0]
        if project_name_value.text:
            project_name = project_name_value.text

    return project_name


def set_unix_slash(win_path):
    """
    Set windows path to unix style path

    :param win_path: windows style path
    :type win_path: str
    :return: unix style path
    :rtype: str
    """
    unix_path = win_path.strip().replace('\\', '/')

    return unix_path


def set_native_slash(raw_path):
    """
    Set native slash

    :param raw_path: any style path
    :type raw_path: str
    :return: unix style path
    :rtype: str
    """

    to_replace = ''
    if os.path.sep == '\\':
        to_replace = '/'
    if os.path.sep == '/':
        to_replace = '\\'

    return raw_path.strip().replace(to_replace, os.path.sep)


def get_mount_point(path):
    folders = path.split(os.path.sep)
    mount_point = ''
    for folder in folders:
        if not folder:
            mount_point += '/'
            continue
        mount_point += '{}/'.format(folder)
        if os.path.ismount(mount_point):
            break
    return mount_point


def check_for_relative_in_path(context, path, remove_relative=True):
    """
    Return path by adding CMake variable or current path prefix, to remove relative

    :param context: the context of converter
    :type context: Context
    :param path: original path
    :type path: str
    :param remove_relative: flag
    :type remove_relative: bool
    :return: formatted path without relative
    :rtype: str
    """

    path_has_drive_path = path[1:2] == ':'
    if path_has_drive_path:
        message(context, 'Found absolute path : {}'.format(path), 'warn')
        return path

    path_starts_with_variable = path[0:1] == '$'
    if not path_starts_with_variable and remove_relative:
        # add current directory for relative path (CMP0021)
        path = '${CMAKE_CURRENT_SOURCE_DIR}/' + path

    return path


def make_os_specific_shell_path(output):
    variables_to_replace = {
        '$(SolutionDir)': '${CMAKE_SOURCE_DIR}/',
        '$(ProjectDir)': '${CMAKE_CURRENT_SOURCE_DIR}/',
        '$(OutDir)': '${OUTPUT_DIRECTORY}',
        '$(TargetPath)': '$<TARGET_FILE:${PROJECT_NAME}>',
    }
    for var in variables_to_replace:
        if var in output:
            output = output.replace(var, '$<SHELL_PATH:{0}>'.format(variables_to_replace[var]))

    return output


def resolve_path_variables_of_vs(context, path_with_vars):
    path_with_vars = path_with_vars.replace('$(ProjectDir)', './')
    path_with_vars = path_with_vars.replace('$(SolutionDir)', context.solution_path)
    return path_with_vars


def get_dir_name_with_vars(context, path):
    del context
    dirname = os.path.dirname(path)
    file_name = os.path.basename(path)
    if not dirname:
        dirname_pattern = re.compile(r'(\$\{.*dir[^\}]*\})', re.IGNORECASE)
        m = dirname_pattern.match(path)
        if m:
            dirname = m.group(0)

        file_name = path.replace(dirname, '')

    file_name, _ = os.path.splitext(os.path.basename(file_name))

    return dirname, file_name


def replace_vs_var_with_cmake_var(context, var):
    variables_to_replace = {
        '$(SolutionDir)': '${CMAKE_SOURCE_DIR}\\',
        '$(Platform)': '${CMAKE_VS_PLATFORM_NAME}',
        '$(PlatformName)': '${CMAKE_VS_PLATFORM_NAME}',
        '$(Configuration)': '$<CONFIG>',
        '$(ConfigurationName)': '$<CONFIG>',
        '$(ProjectDir)': '${CMAKE_CURRENT_SOURCE_DIR}\\',
        '$(ProjectName)': '${PROJECT_NAME}',
        '$(RootNamespace)': '${ROOT_NAMESPACE}',
        '$(OutDir)': '${OUTPUT_DIRECTORY}',
        '$(OUTDIR)': '${OUTPUT_DIRECTORY}',
        '$(IntDir)': '${CMAKE_CURRENT_BINARY_DIR}\\${CMAKE_CFG_INTDIR}\\',
        '$(INTDIR)': '${CMAKE_CURRENT_BINARY_DIR}\\${CMAKE_CFG_INTDIR}\\',
        '$(TargetDir)': '${OUTPUT_DIRECTORY}',
        '$(TargetName)': '${TARGET_NAME}',
        '$(TargetFileName)': '$<TARGET_FILE_NAME:${TARGET_NAME}>',
        '$(TargetPath)': '$<TARGET_FILE:${PROJECT_NAME}>',
    }
    if var in variables_to_replace:
        return variables_to_replace[var]

    var_name = var[2:-1]
    cmake_env_var = '$ENV{{{}}}'.format(var_name)
    if var_name not in os.environ:
        message(context, 'Unknown variable: {}, trying {}'.format(var, cmake_env_var), 'warn')
    return cmake_env_var


def replace_vs_vars_with_cmake_vars(context, output):

    var_ex = re.compile(r'(\$\(.*?\))')

    vars_list = var_ex.findall(output)
    for var in vars_list:
        replace_with = replace_vs_var_with_cmake_var(context, var)
        output = output.replace(var, replace_with)

    return output


def cleaning_output(context, output):
    """
    Clean Output string by remove VS Project Variables

    :param context: the context of converter
    :type context: Context
    :param output: Output to clean
    :type output: str
    :return: clean output
    :rtype: str
    """

    output = replace_vs_vars_with_cmake_vars(context, output)

    output = set_unix_slash(output)

    return output


def insensitive_glob(path):
    drive, path = os.path.splitdrive(path)

    def either(c):
        return '[{}{}]'.format(c.lower(), c.upper()) if c.isalpha() else c
    pattern = drive + ''.join(map(either, path))
    return glob.glob(pattern)


def get_actual_filename(context, name):
    """
    Return actual filename from given name if file iis found, else return None

    :param context: the context of converter
    :type context: Context
    :param name: name of file
    :type name: str
    :return: None | str
    :rtype: None | str
    """

    res = insensitive_glob(name)
    if not res:
        # File not found
        message(context, 'file or path "{0}" not found.'.format(name), 'warn')
        return None

    return res[0]


def normalize_path(context, working_path, path_to_normalize, remove_relative=True):
    """
    Normalize path from working path

    :param context: the context of converter
    :type context: Context
    :param working_path: current working path
    :type working_path: str
    :param path_to_normalize: path to be normalized
    :type path_to_normalize: str
    :param remove_relative: remove relative from path flag
    :type remove_relative: bool
    :return: normalized path
    :rtype: str
    """

    joined_path = set_native_slash(
        os.path.join(working_path, ntpath.normpath(path_to_normalize.strip()))
    )
    normal_path = os.path.normpath(joined_path)
    actual_path_name = get_actual_filename(context, normal_path)
    if actual_path_name is None:
        message(
            context,
            'getting actual filesystem name failed : "{}"'.format(normal_path),
            'warn1'
        )
        actual_path_name = normal_path
    normal_path = set_unix_slash(os.path.relpath(actual_path_name, working_path))
    normal_path = check_for_relative_in_path(context, normal_path, remove_relative)

    return normal_path


def prepare_build_event_cmd_line_for_cmake(context, build_event):
    cmake_build_event = make_os_specific_shell_path(build_event)
    cmake_build_event = replace_vs_vars_with_cmake_vars(context, cmake_build_event)
    cmake_build_event = cmake_build_event.replace('\\', '\\\\')
    return cmake_build_event


def message(context, text, status):  # pragma: no cover
    """
    Displays a message while the script is running

    :param context: the context of converter
    :type context: Context
    :param text: content of the message
    :type text: str
    :param status: level of the message (change color)
    :type status: str
    """

    current_time = time.time()
    delta_time = current_time - context.time0
    dt = '{0:f} '.format(delta_time)

    message_begin = dt
    if context.project_number:
        message_begin = message_begin + '{0}> '.format(context.project_number)

    if None not in context.current_setting:
        text = '{0} : {1}'.format(context.current_setting, text)

    if status == 'error':
        print(message_begin + 'ERR  : ' + FAIL + text + ENDC)
    elif 'warn' in status:
        if status == 'warn':
            status += '1'
        message_warning_level = int(status[-1])
        if message_warning_level <= context.warn_level:
            print(message_begin + 'WARN : ' + WARN + text + ENDC)
            context.warnings_count += 1
    elif status == 'ok':
        print(message_begin + 'OK   : ' + OK + text + ENDC)
    elif status == 'done':
        print(message_begin + DONE + text + ENDC)
    else:
        if context.verbose:
            print(message_begin + 'INFO : ' + text)


def get_comment(text):
    line_length = 80
    title_line = ''

    for _ in range(0, line_length):
        title_line = '{0}{1}'.format(title_line, '#')

    comment = title_line + '\n'
    comment += '# {0}\n'.format(text)
    comment += title_line + '\n'
    return comment


def write_comment(cmake_file, text):
    """
    Write formated comment in given file wrapper

    :param text: text in middle of title
    :type text: str
    :param cmake_file: CMakeLists.txt IO wrapper
    :type cmake_file: _io.TextIOWrapper
    """

    cmake_file.write(get_comment(text))


def write_arch_types(cmake):
    write_comment(
        cmake,
        'Set target arch type if empty. Visual studio solution generator provides it.'
    )
    cmake.write('if(NOT CMAKE_VS_PLATFORM_NAME)\n')
    cmake.write('    set(CMAKE_VS_PLATFORM_NAME "x64")\n')
    cmake.write('endif()\n')
    cmake.write('message(\"${CMAKE_VS_PLATFORM_NAME} architecture in use\")\n\n')


def write_use_package_stub(cmake):
    write_comment(cmake, 'Nuget packages function stub.')
    cmake.write('function(use_package TARGET PACKAGE VERSION)\n')
    cmake.write('    message(WARNING "No implementation of use_package. Create yours.")\n')
    cmake.write('endfunction()\n\n')


def make_cmake_literal(context, input_str):

    output_str = re.sub(r'[^0-9a-zA-Z_./\-+]', '', input_str)

    if input_str != output_str:
        message(context, 'check conversion "{}" -> "{}"'.format(input_str, output_str), 'warn')
    return output_str



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2017:
#   Matthieu Estrada, ttamalfor@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import sys

try:
    from setuptools import setup, find_packages
except Exception:
    sys.exit("Error: missing python-setuptools library")

try:
    python_version = sys.version_info
except Exception:
    python_version = (1, 5)
if python_version < (3, 4):
    sys.exit(
        "This application requires a minimum of Python 3.4 !"
        " Please update your Python version."
    )

from cmake_converter import __description__, __version__, __license__, __author__, __project_url__
from cmake_converter import __name__ as __pkg_name__

# Requirements
install_requires = [
    'lxml',
    'colorama',
]


setup(
    name=__pkg_name__,
    version=__version__,

    license=__license__,

    # metadata for upload to PyPI
    author=__author__,
    author_email="liavonlida@gmail.com ,ttamalfor@gmail.com",
    keywords="cmake vcxproj vfproj visual cpp fortran CMakeLists",
    url=__project_url__,
    description=__description__,
    long_description=open('README.rst').read(),

    zip_safe=False,

    packages=find_packages(),
    include_package_data=True,

    install_requires=install_requires,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Software Development :: Compilers',
        'Topic :: Software Development :: Libraries'
    ],

    entry_points={
        'console_scripts': [
            'cmake-converter = cmake_converter.main:main'
        ],
    }

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

from cmake_converter.project_variables import ProjectVariables
from cmake_converter.utils import write_property_of_settings, message


class VFProjectVariables(ProjectVariables):
    """
         Class who defines all the CMake variables to be used by the Fortran project
    """

    def set_output_dir(self, context, attr_name, output_dir, node):
        del attr_name, node

        self.set_output_dir_impl(context, output_dir)

    def set_output_file(self, context, flag_name, output_file, node):
        del flag_name, node

        self.set_output_file_impl(context, output_file)

    def set_module_dir(self, context, flag_name, output_file, node):
        del flag_name, node

        self.set_path_and_name_from_node(
            context,
            'Fortran module directory',
            output_file,
            'Fortran_MODULE_DIRECTORY',
            'Fortran_MODULE_NAME'   # no support in CMake
        )

        message(
            context,
            'Fortran Module Directory will be ignored due lack of regex support at CMake\n'
            'See walk-around at CMake/DefaultFortran.cmake', 'warn3'
        )

    def set_import_library(self, context, flag_name, import_library, node):
        del flag_name, node

        self.set_path_and_name_from_node(
            context,
            'Import library',
            import_library,
            'ARCHIVE_OUTPUT_DIRECTORY',
            'ARCHIVE_OUTPUT_NAME'
        )

    def set_program_database_file(self, context, flag_name, program_database_file, node):
        del flag_name, node
        self.set_path_and_name_from_node(
            context,
            'Program database',
            program_database_file,
            'PDB_OUTPUT_DIRECTORY',
            'PDB_NAME'
        )

    @staticmethod
    def write_target_outputs(context, cmake_file):
        ProjectVariables.write_target_outputs(context, cmake_file)

        # No support of Regex for Fortran_MODULE_DIRECTORY at CMake 3.13
        # write_property_of_settings(
        #     cmake_file, context.settings,
        #     context.sln_configurations_map,
        #     begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
        #     end_text=')',
        #     property_name='Fortran_MODULE_DIRECTORY',
        #     write_setting_property_func=ProjectVariables.write_target_property
        # )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    CMakeConverter

    This module is a command line program who allow you to convert .vcxproj file
    to CMakeLists.txt file.
"""

# Application version and manifest
VERSION = (2, 0, 0, 'dev15072019')
__application__ = u"CMake-Converter"
__short_version__ = '.'.join((str(each) for each in VERSION[:2]))
__version__ = '.'.join((str(each) for each in VERSION[:4]))
__author__ = u"Liavonau Pavel, Estrada Matthieu"
__copyright__ = u"2016-2019 - %s" % __author__
__license__ = u"GNU Affero General Public License, version 3"
__description__ = u"CMake converter for Visual Studio projects"
__releasenotes__ = u"CMake converter for Visual Studio projects"
__project_url__ = "https://github.com/pavelliavonau/cmakeconverter"
__doc_url__ = "https://github.com/pavelliavonau/cmakeconverter"

# Application Manifest
manifest = {
    'name': __application__,
    'version': __version__,
    'author': __author__,
    'description': __description__,
    'copyright': __copyright__,
    'license': __license__,
    'release': __releasenotes__,
    'url': __project_url__,
    'doc': __doc_url__
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Pavel Liavonau, liavonlida@gmail.com
#   Matthieu Estrada, ttamalfor@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

from cmake_converter.parser import Parser, StopParseException


class VFParser(Parser):

    def __init__(self):
        Parser.__init__(self)
        self.common_diagnostics_value = None
        self.common_runtime_checks_value = None
        self.source_group = ''

    def get_node_handlers_dict(self, context):
        node_handlers = {
            'Platforms': self.do_nothing_node_stub,
            'Configurations': self.__parse_configurations,
            'Configuration': self.__parse_configuration,
            'FileConfiguration': self.__parse_configuration,
            'Tool': self._parse_nodes,
            'Files': self.__parse_files,
            'File': self.do_nothing_node_stub,
            'Filter': self.__parse_filter,
            'Globals': self.do_nothing_node_stub,
        }
        return node_handlers

    def get_attribute_handlers_dict(self, context):
        attributes_handlers = {}

        attributes_handlers.update(
            dict.fromkeys(context.flags.flags_handlers.keys(), context.flags.set_flag)
        )

        attributes_handlers.update({
            'Configuration_Name': self.__parse_configuration_name,
            'FileConfiguration_Name': self.__parse_configuration_name,
            'Configuration_TargetName': self.__parse_target_name,
            'Configuration_OutputDirectory': context.variables.set_output_dir,
            'Configuration_IntermediateDirectory': self.__parse_configuration_intermediate_dir,
            'Configuration_DeleteExtensionsOnClean': self.do_nothing_attr_stub,
            'Configuration_ConfigurationType': self.__parse_configuration_type,
            'Tool_Name': self.__parse_tool_name,
            'VFFortranCompilerTool_PreprocessorDefinitions': self.__parse_preprocessor_definitions,
            'VFFortranCompilerTool_AdditionalIncludeDirectories':
                self.__parse_additional_include_directories,
            'VFFortranCompilerTool_Diagnostics': self.__parse_diagnostics,
            'VFFortranCompilerTool_WarnDeclarations': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnUnusedVariables': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnIgnoreLOC': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnTruncateSource': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnInterfaces': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnUnalignedData': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_WarnUncalled': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_SuppressUsageMessages': self.__parse_concrete_diagnostics,
            'VFFortranCompilerTool_RuntimeChecks': self.__parse_runtime_checks,
            'VFFortranCompilerTool_NullPointerCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_BoundsCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_UninitializedVariablesCheck':
                self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_DescriptorDataTypeCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_DescriptorDataSizeCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_ArgTempCreatedCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_StackFrameCheck': self.__parse_concrete_runtime_checks,
            'VFFortranCompilerTool_ModulePath': context.variables.set_module_dir,
            'VFFortranCompilerTool_ObjectFile': self.do_nothing_attr_stub,  # TODO?
            'VFFortranCompilerTool_AssemblerListingLocation': self.do_nothing_attr_stub,  # TODO?
            'VFFortranCompilerTool_PdbFile': self.do_nothing_attr_stub,  # TODO?
            'VFFortranCompilerTool_ParallelizerDiagLevel': self.do_nothing_attr_stub,  # obsolete?
            'VFFortranCompilerTool_VectorizerDiagLevel': self.do_nothing_attr_stub,  # obsolete?
            'VFLinkerTool_OutputFile': context.variables.set_output_file,
            'VFLibrarianTool_OutputFile': context.variables.set_output_file,
            'VFLinkerTool_ImportLibrary': context.variables.set_import_library,
            'VFLinkerTool_ProgramDatabaseFile': context.variables.set_program_database_file,
            'VFLibrarianTool_ImportLibrary': context.variables.set_import_library,
            'VFLinkerTool_AdditionalDependencies':
                context.dependencies.set_target_additional_dependencies,
            'VFLibrarianTool_AdditionalDependencies':
                context.dependencies.set_target_additional_dependencies,
            'VFLinkerTool_AdditionalLibraryDirectories':
                context.dependencies.set_target_additional_library_directories,
            'VFLibrarianTool_AdditionalLibraryDirectories':
                context.dependencies.set_target_additional_library_directories,
            'VFResourceCompilerTool_PreprocessorDefinitions':
                self.__parse_preprocessor_definitions,
            'VFResourceCompilerTool_Culture':
                self.do_nothing_attr_stub,
            'VFResourceCompilerTool_ResourceOutputFileName':
                self.do_nothing_attr_stub,  # TODO?
            'VFPreBuildEventTool_ExcludedFromBuild': self.do_nothing_attr_stub,
            'VFPreBuildEventTool_CommandLine':
                context.dependencies.set_target_pre_build_events,
            'VFPreLinkEventTool_ExcludedFromBuild': self.do_nothing_attr_stub,
            'VFPreLinkEventTool_CommandLine':
                context.dependencies.set_target_pre_link_events,
            'VFPostBuildEventTool_ExcludedFromBuild': self.do_nothing_attr_stub,
            'VFPostBuildEventTool_CommandLine':
                context.dependencies.set_target_post_build_events,
            'VFCustomBuildTool_ExcludedFromBuild': self.do_nothing_attr_stub,
            'VFCustomBuildTool_CommandLine':
                context.dependencies.set_custom_build_step,
            'VFCustomBuildTool_Description': self.do_nothing_attr_stub,
            'VFCustomBuildTool_Outputs': self.do_nothing_attr_stub,
            'File_RelativePath': self.__parse_file_relative_path,
            'Filter_Name': self.do_nothing_attr_stub,
            'Filter_Filter': self.do_nothing_attr_stub,  # TODO?
        })
        return attributes_handlers

    def parse(self, context):
        tree = context.vcxproj['tree']
        root = tree.getroot()
        self._parse_nodes(context, root)
        context.files.apply_files_to_context(context)
        context.dependencies.set_target_references(context)

    def __parse_configurations(self, context, configurations_node):
        self._parse_nodes(context, configurations_node)

    def __parse_configuration(self, context, configuration_node):

        context.flags.prepare_context_for_flags(context)
        self._parse_nodes(context, configuration_node)
        context.flags.apply_flags_to_context(context)

    def __parse_configuration_name(self, context, attr_name, configuration_name, node):
        del attr_name

        setting = tuple(configuration_name.split('|'))

        if setting not in context.configurations_to_parse:
            context.current_setting = (None, None)
            raise StopParseException()

        context.current_setting = setting
        self.common_diagnostics_value = None
        self.common_runtime_checks_value = None
        context.utils.init_context_current_setting(context)
        self.reset_current_setting_after_parsing_node(node)

    @staticmethod
    def __parse_target_name(context, attr_name, target_name_value, node):
        del attr_name, node

        context.variables.set_target_name(context, target_name_value)

    @staticmethod
    def __parse_configuration_intermediate_dir(context, attr_name, intermediate_dir_value, node):
        pass

    @staticmethod
    def __parse_configuration_type(context, attr_name, configuration_type_value, node):
        del attr_name, node

        configuration_type_value = configuration_type_value.replace('type', '')
        context.settings[context.current_setting]['target_type'] = configuration_type_value

    @staticmethod
    def __parse_tool_name(context, attr_name, tool_name_value, tool_node):
        del attr_name, context

        if tool_name_value in ['VFMidlTool', 'VFManifestTool']:
            raise StopParseException()
        tool_node.tag = tool_name_value

    @staticmethod
    def __parse_preprocessor_definitions(context, flag_name, preprocessor_definitions_value, node):
        del flag_name, node
        for define in preprocessor_definitions_value.split(';'):
            define = define.strip()
            context.settings[context.current_setting]['defines'].append(define)

    @staticmethod
    def __parse_additional_include_directories(
            context,
            flag_name,
            additional_include_directories_value,
            node
    ):
        del flag_name, node

        context.dependencies.set_additional_include_directories(
            additional_include_directories_value,
            context.current_setting,
            context
        )

    def __parse_diagnostics(self, context, attr_name, attr_value, node):
        context.flags.set_flag(context, attr_name, attr_value, node)
        self.common_diagnostics_value = attr_value

    def __parse_concrete_diagnostics(self, context, attr_name, attr_value, node):
        if self.common_diagnostics_value is None:
            context.flags.set_flag(context, attr_name, attr_value, node)

    def __parse_runtime_checks(self, context, attr_name, attr_value, node):
        context.flags.set_flag(context, attr_name, attr_value, node)
        self.common_runtime_checks_value = attr_value

    def __parse_concrete_runtime_checks(self, context, attr_name, attr_value, node):
        if self.common_runtime_checks_value is None:
            context.flags.set_flag(context, attr_name, attr_value, node)

    def __parse_files(self, context, filter_node):
        context.files.init_file_lists_for_include_paths(context)
        self._parse_nodes(context, filter_node)

    def __parse_file_relative_path(self, context, attr_name, attr_value, file_node):
        del attr_name, attr_value

        source_group = ''
        parent = file_node.getparent()
        if Parser.strip_namespace(parent.tag) == 'Filter':
            source_group = parent.attrib['Name']
        file_context = context.files.add_file_from_node(
            context,
            files_container=context.sources,
            file_node=file_node,
            file_node_attr='RelativePath',
            source_group=source_group
        )
        self._parse_nodes(file_context, file_node)
        context.warnings_count += file_context.warnings_count

    def __parse_filter(self, context, filter_node):
        self._parse_nodes(context, filter_node)

    def __parse_filter_name(self, context, attr_name, attr_value, file_node):
        del context, attr_name, file_node

        self.source_group = attr_value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    Flags
    =====
     Manage compilation flags of project
"""

from cmake_converter.utils import message, write_comment
from cmake_converter.utils import write_property_of_settings

cl_flags = 'cl_flags'               # MSVC compile flags (Windows only)
ln_flags = 'ln_flags'               # MSVC link flags (Windows only)
ifort_cl_win = 'ifort_cl_win'       # ifort compile flags (Windows)
ifort_cl_unix = 'ifort_cl_unix'     # ifort compile flags (Unix)
ifort_ln_win = 'ifort_ln_win'       # ifort link flags for windows
ifort_ln_unix = 'ifort_ln_unix'     # ifort link flags for unix
defines = 'defines'                 # compile definitions (cross platform)
default_value = 'default_value'


class Flags:
    """
        Class who manage flags of projects
    """

    @staticmethod
    def __write_defines_for_files(cmake_file,
                                  property_indent,
                                  config_condition_expr,
                                  property_value,
                                  width,
                                  **kwargs):
        del width, kwargs
        config = ''
        if config_condition_expr:
            config = config_condition_expr.replace('$<CONFIG:', '_')
            config = config.replace('>', '')

        cmake_file.write(
            '{0}    COMPILE_DEFINITIONS{1} "{2}"\n'
            .format(property_indent, config.upper(), ';'.join(property_value))
        )

    def write_defines(self, context, cmake_file):
        write_comment(cmake_file, 'Compile definitions')
        write_property_of_settings(
            cmake_file, context.settings, context.sln_configurations_map,
            begin_text='target_compile_definitions(${PROJECT_NAME} PRIVATE',
            end_text=')',
            property_name=defines,
            separator=';\n',
            in_quotes=True
        )
        for file in context.file_contexts:
            write_property_of_settings(
                cmake_file,
                context.file_contexts[file].settings,
                context.sln_configurations_map,
                begin_text='set_source_files_properties({0} PROPERTIES'.format(file),
                end_text=')',
                property_name=defines,
                write_setting_property_func=self.__write_defines_for_files
            )
        cmake_file.write('\n')

    @staticmethod
    def __write_compile_flags(context, cmake_file, compiler_flags_key):
        write_property_of_settings(
            cmake_file, context.settings, context.sln_configurations_map,
            begin_text='target_compile_options(${PROJECT_NAME} PRIVATE',
            end_text=')',
            property_name=compiler_flags_key,
            separator=';\n',
            indent='    '
        )
        for file in context.file_contexts:
            file_cl_var = 'FILE_CL_OPTIONS'
            text = write_property_of_settings(
                cmake_file, context.file_contexts[file].settings,
                context.sln_configurations_map,
                begin_text='string(CONCAT {0}'.format(file_cl_var),
                end_text=')',
                property_name=compiler_flags_key,
                indent='    ',
                in_quotes=True
            )
            if text:
                cmake_file.write(
                    '    source_file_compile_options({0} ${{{1}}})\n'
                    .format(file, file_cl_var))

    @staticmethod
    def __write_link_flags(context, cmake_file, linker_flags_key):
        write_property_of_settings(
            cmake_file, context.settings, context.sln_configurations_map,
            begin_text='target_link_options(${PROJECT_NAME} PRIVATE',
            end_text=')',
            property_name=linker_flags_key,
            separator=';\n',
            indent='    '
        )

    @staticmethod
    def write_compile_and_link_flags(context, cmake_file, **kwargs):
        """

        :param context:
        :param cmake_file:
        :param kwargs:
        :return:
        """
        os_check_str = kwargs['os_check_str']
        compiler_check_str = kwargs['compiler_check_str']
        compiler_flags_key = kwargs['compiler_flags_key']
        linker_flags_key = kwargs['linker_flags_key']

        and_os_str = ''
        if os_check_str:
            and_os_str = ' AND {0}'.format(os_check_str)
        cmake_file.write('if({0}{1})\n'.format(compiler_check_str, and_os_str))

        Flags.__write_compile_flags(context, cmake_file, compiler_flags_key)
        Flags.__write_link_flags(context, cmake_file, linker_flags_key)

        cmake_file.write('endif()\n\n')

    def write_flags(self, context, cmake_file):
        """
        Get and write Preprocessor Macros definitions

        :param context: converter Context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """
        is_msvc = False
        for setting in context.settings:
            if cl_flags in context.settings[setting]:
                if context.settings[setting][cl_flags]:
                    is_msvc = True
                    break

        is_ifort = False
        for setting in context.settings:
            if ifort_cl_win in context.settings[setting]:
                if context.settings[setting][ifort_cl_win]:
                    is_ifort = True
                    break

        write_comment(cmake_file, 'Compile and link options')
        if is_msvc:
            self.write_compile_and_link_flags(
                context, cmake_file,
                os_check_str=None,
                compiler_check_str='MSVC',
                compiler_flags_key=cl_flags,
                linker_flags_key=ln_flags,
            )

        if is_ifort:
            self.write_compile_and_link_flags(
                context, cmake_file,
                os_check_str='WIN32',
                compiler_check_str='${CMAKE_Fortran_COMPILER_ID} STREQUAL "Intel"',
                compiler_flags_key=ifort_cl_win,
                linker_flags_key=ifort_ln_win
            )
            self.write_compile_and_link_flags(
                context, cmake_file,
                os_check_str='UNIX',
                compiler_check_str='${CMAKE_Fortran_COMPILER_ID} STREQUAL "Intel"',
                compiler_flags_key=ifort_cl_unix,
                linker_flags_key=ifort_ln_unix
            )

    @staticmethod
    def write_target_headers_only_artifact(context, cmake_file):
        """
        Add a dummy target to given CMake file

        :param context: the context of converter
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        message(context, 'CMake will show fake custom Library.', '')
        cmake_file.write('add_custom_target(${PROJECT_NAME} SOURCES ${ALL_FILES})\n\n')

    def write_use_pch_function(self, context, cmake_file):
        pass

    @staticmethod
    def write_target_artifact(context, cmake_file):
        """
        Add Library or Executable target

        :param context: converter Context
        :type context: Context
        :param cmake_file: CMakeLIsts.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        configuration_type = None
        for s in context.settings:
            if None in s:
                continue
            configuration_type = context.settings[s]['target_type']
            if configuration_type:
                break
        if configuration_type:
            if configuration_type == 'DynamicLibrary':
                cmake_file.write('add_library(${PROJECT_NAME} SHARED')
                message(context, 'CMake will build a SHARED Library.', '')
            elif configuration_type == 'StaticLibrary':  # pragma: no cover
                cmake_file.write('add_library(${PROJECT_NAME} STATIC')
                message(context, 'CMake will build a STATIC Library.', '')
            else:  # pragma: no cover
                cmake_file.write('add_executable(${PROJECT_NAME}')
                message(context, 'CMake will build an EXECUTABLE.', '')
            cmake_file.write(' ${ALL_FILES})\n')

        if context.solution_folder:
            cmake_file.write(
                'set_target_properties(${{PROJECT_NAME}} PROPERTIES FOLDER "{}")\n'.format(
                    context.solution_folder
                )
            )

        cmake_file.write('\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import re
import os
from collections import OrderedDict

from cmake_converter.flags import Flags, defines, cl_flags, default_value, ln_flags
from cmake_converter.utils import take_name_from_list_case_ignore, normalize_path, cleaning_output
from cmake_converter.utils import set_unix_slash, message, replace_vs_vars_with_cmake_vars


class NodeStub:
    def __init__(self, flag_name):
        self.text = ''
        self.tag = flag_name


class CPPFlags(Flags):
    """
        Class who check and create compilation flags
    """

    def __init__(self):
        self.flags = {}
        self.unicode_defines = {}
        self.flags_handlers = OrderedDict([
            # compilation cl_flags
            ('ConformanceMode', self.__set_conformance_mode),
            ('SupportJustMyCode', self.__set_support_just_my_code),
            ('LanguageStandard', self.__set_language_standard),
            ('UseDebugLibraries', self.__set_use_debug_libraries),
            ('WholeProgramOptimization', self.__set_whole_program_optimization),
            ('MinimalRebuild', self.__set_minimal_rebuild),
            ('Optimization', self.__set_optimization),
            ('InlineFunctionExpansion', self.__set_inline_function_expansion),
            ('IntrinsicFunctions', self.__set_intrinsic_functions),
            ('SDLCheck', self.__set_sdl_check),
            ('StringPooling', self.__set_string_pooling),
            ('EnableFiberSafeOptimizations', self.__set_enable_fiber_safe_optimizations),
            ('BasicRuntimeChecks', self.__set_basic_runtime_checks),
            ('ShowIncludes', self.__set_show_includes),
            ('CompileAsManaged', self.__set_compile_as_managed),
            ('EnableEnhancedInstructionSet', self.__set_enable_enhanced_instruction_set),
            ('OmitFramePointers', self.__set_omit_frame_pointers),
            ('CallingConvention', self.__set_calling_convention),
            ('RuntimeLibrary', self.__set_runtime_library),
            ('FunctionLevelLinking', self.__set_function_level_linking),
            ('WarningLevel', self.__set_warning_level),
            ('SuppressStartupBanner', self.__set_suppress_startup_banner),
            ('FloatingPointExceptions', self.__set_floating_point_exceptions),
            ('TreatWarningAsError', self.__set_warning_as_errors),
            ('DebugInformationFormat', self.__set_debug_information_format),
            ('AssemblerListingLocation', self.__set_assembler_listing_location),
            ('AssemblerOutput', self.__set_assembler_output),
            ('ObjectFileName', self.__set_object_file_name),
            ('FavorSizeOrSpeed', self.__set_favour_size_or_speed),
            ('CompileAs', self.__set_compile_as),
            ('FloatingPointModel', self.__set_floating_point_model),
            ('StructMemberAlignment', self.__set_struct_member_alignment),
            ('RuntimeTypeInfo', self.__set_runtime_type_info),
            ('CLRSupport', self.__set_compile_as_managed),
            ('DisableSpecificWarnings', self.__set_disable_specific_warnings),
            ('CompileAdditionalOptions', self.__set_compile_additional_options),
            ('ExceptionHandling', self.__set_exception_handling),
            ('BufferSecurityCheck', self.__set_buffer_security_check),
            ('ControlFlowGuard', self.__set_control_flow_guard),
            ('DiagnosticsFormat', self.__set_diagnostics_format),
            ('DisableLanguageExtensions', self.__set_disable_language_extensions),
            ('TreatWChar_tAsBuiltInType', self.__set_treatwchar_t_as_built_in_type),
            ('ForceConformanceInForLoopScope', self.__set_force_conformance_in_for_loop_scope),
            ('RemoveUnreferencedCodeData', self.__set_remove_unreferenced_code_data),
            ('OpenMPSupport', self.__set_openmp_support),
            ('PrecompiledHeader', self.__set_precompiled_header),
            ('PrecompiledHeaderFile', self.__set_precompiled_header_file),
            #   linking ln_flags
            ('FixedBaseAddress', self.__set_fixed_base_address),
            ('StackReserveSize', self.__set_stack_reserve_size),
            ('GenerateDebugInformation', self.__set_generate_debug_information),
            ('TargetMachine', self.__set_target_machine),
            ('ImageHasSafeExceptionHandlers', self.__set_image_has_safe_exception_handlers),
            ('SubSystem', self.__set_sub_system),
            ('OptimizeReferences', self.__set_optimize_references),
            ('EnableCOMDATFolding', self.__set_enable_comdat_folding),
            ('Profile', self.__set_profile),
            ('DataExecutionPrevention', self.__set_data_execution_prevention),
            ('RandomizedBaseAddress', self.__set_randomized_base_address),
            ('LinkIncremental', self.__set_link_incremental),
            ('IgnoreEmbeddedIDL', self.__set_ignore_embedded_idl),
            ('AssemblyDebug', self.__set_assembly_debug),
            ('LinkAdditionalOptions', self.__set_link_additional_options),
        ])

    def __set_default_flag(self, context, flag_name):
        self.flags[context.current_setting][flag_name] = {}
        stub = NodeStub(flag_name)
        self.set_flag(context, stub)

    def __set_default_flags(self, context):
        message(context, '== start making default flags ==', '')
        if context.current_setting not in self.flags:
            self.flags[context.current_setting] = {}
        for flag_name in self.flags_handlers:
            self.__set_default_flag(context, flag_name)
        message(context, '== end making default flags ==', '')

    def set_defines(self, context, defines_node):
        """
        Defines preprocessor definitions in current settings

        """
        for define in defines_node.text.split(";"):
            define = define.strip()
            if define != '%(PreprocessorDefinitions)':
                define = replace_vs_vars_with_cmake_vars(context, define)
                define = define.replace('\\', '\\\\')
                define = define.replace('"', '\\"')
                context.settings[context.current_setting][defines].append(define)
        if context.current_setting in self.unicode_defines:
            for define in self.unicode_defines[context.current_setting]:
                context.settings[context.current_setting][defines].append(define)
        message(context, 'PreprocessorDefinitions : {}'.format(
            context.settings[context.current_setting][defines]
        ), '')

    def set_character_set(self, context, character_set_node):
        if 'Unicode' in character_set_node.text:
            self.unicode_defines[context.current_setting] = ['UNICODE', '_UNICODE']
        if 'MultiByte' in character_set_node.text:
            self.unicode_defines[context.current_setting] = ['_MBCS']

    @staticmethod
    def __set_precompiled_header(context, flag_name, node):
        """
        Add precompiled headers to settings

        :param context: converter Context
        :type context: Context
        :param flag_name:
        :param node:
        """
        del context, flag_name
        if isinstance(node, NodeStub):
            return {}   # default pass

        precompiled_header_values = {
            'Use': {'PrecompiledHeader': 'Use'},
            'NotUsing': {'PrecompiledHeader': 'NotUsing',
                         cl_flags: '/Y-'},
            'Create': {'PrecompiledHeader': 'Create'},
            default_value: {'PrecompiledHeader': 'NotUsing',
                            cl_flags: '/Y-'}
        }
        return precompiled_header_values

    @staticmethod
    def __set_precompiled_header_file(context, flag_name, node):
        del flag_name

        pch_header_file = 'stdafx.h'  # default

        if node.text:
            pch_header_file = node.text

        context.settings[context.current_setting]['PrecompiledHeaderFile'] = pch_header_file

    @staticmethod
    def __define_pch_paths(context, setting):
        pch_header_name = context.settings[setting]['PrecompiledHeaderFile']
        found = False
        founded_pch_h_path = ''
        for headers_path in context.headers:
            for header in context.headers[headers_path]:
                if header.lower() == pch_header_name.lower():
                    found = True
                    founded_pch_h_path = headers_path
                    pch_header_name = header
                if found:
                    break
            if found:
                break
        if founded_pch_h_path:
            founded_pch_h_path += '/'
        pch_header_path = founded_pch_h_path + pch_header_name

        pch_source_name = pch_header_name.replace('.h', '.cpp')
        real_pch_cpp = ''
        real_pch_cpp_path = ''
        if founded_pch_h_path in context.sources:
            real_pch_cpp = take_name_from_list_case_ignore(
                context,
                context.sources[founded_pch_h_path], pch_source_name
            )
            real_pch_cpp_path = founded_pch_h_path
        else:
            for src_path in context.sources:
                for src in context.sources[src_path]:
                    if pch_source_name == src:
                        real_pch_cpp = take_name_from_list_case_ignore(
                            context,
                            context.sources[src_path], src
                        )
                        real_pch_cpp_path = src_path
        if real_pch_cpp_path:
            real_pch_cpp_path += '/'
        pch_source_path = real_pch_cpp_path + real_pch_cpp

        return pch_header_path, pch_source_path

    def define_pch_cpp_file(self, context):
        pch_header_path = ''
        pch_source_path = ''
        for setting in context.settings:

            if 'Use' not in context.settings[setting]['PrecompiledHeader']:
                continue

            if pch_header_path == '':
                pch_header_path, pch_source_path = self.__define_pch_paths(context, setting)

            context.settings[setting]['PrecompiledHeaderFile'] = pch_header_path
            context.settings[setting]['PrecompiledSourceFile'] = pch_source_path

    def set_flag(self, context, node):
        """
        Set flag helper

        :param context: converter Context
        :type context: Context
        :param node:
        :return:
        """

        if None in context.current_setting:
            return

        flag_name = re.sub(r'{.*\}', '', node.tag)  # strip namespace
        flag_value = node.text

        flag_values = self.flags_handlers[flag_name](context, flag_name, node)

        if flag_values is None:
            return

        values = None
        if default_value in flag_values:
            values = flag_values[default_value]

        if flag_value in flag_values:
            values = flag_values[flag_value]

        flags_message = {}
        if values is not None:
            self.flags[context.current_setting][flag_name] = {}  # reset default values
            for key in values:
                value = values[key]
                self.flags[context.current_setting][flag_name][key] = [value]
                flags_message[key] = value

        if flags_message:
            message(
                context,
                '{0} is {1} '.format(flag_name, flags_message),
                ''
            )

    def prepare_context_for_flags(self, context):
        if context.current_setting not in context.flags.flags:
            context.flags.flags[context.current_setting] = {}
            self.__set_default_flags(context)

    def apply_flags_to_context(self, context):
        context_flags_data_keys = [
            cl_flags,
            ln_flags,
            'PrecompiledHeader',
        ]

        for setting in context.settings:
            self.__apply_generate_debug_information(context, setting)
            self.__apply_link_incremental(context, setting)
            for flag_name in self.flags_handlers:
                for context_flags_data_key in context_flags_data_keys:
                    if context_flags_data_key in self.flags[setting][flag_name]:
                        for value in self.flags[setting][flag_name][context_flags_data_key]:
                            context.settings[setting][context_flags_data_key].append(
                                value
                            )
                    for file in context.file_contexts:
                        file_context = context.file_contexts[file]
                        if setting not in context.file_contexts[file].flags.flags:
                            continue
                        file_flags = context.file_contexts[file].flags.flags[setting]
                        if flag_name not in file_flags:
                            continue
                        if context_flags_data_key in file_flags[flag_name]:
                            for value in file_flags[flag_name][context_flags_data_key]:
                                file_context.settings[setting][context_flags_data_key].append(
                                    value
                                )

    def __apply_generate_debug_information(self, context, setting):
        conf_type = context.settings[setting]['target_type']
        if conf_type and 'StaticLibrary' in conf_type:
            self.flags[setting]['GenerateDebugInformation'][ln_flags] = ''

    def __apply_link_incremental(self, context, setting):
        conf_type = context.settings[setting]['target_type']
        if conf_type and 'StaticLibrary' in conf_type:
            self.flags[setting]['LinkIncremental'][ln_flags] = ''

    @staticmethod
    def __set_whole_program_optimization(context, flag_name, node):
        """
        Set Whole Program Optimization flag: /GL and /LTCG

        """
        del context, flag_name, node
        flag_values = {'true': {ln_flags: '/LTCG', cl_flags: '/GL'}}

        return flag_values

    @staticmethod
    def __set_link_incremental(context, flag_name, node):
        """
        Set LinkIncremental flag: /INCREMENTAL

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/INCREMENTAL'},
            'false': {ln_flags: '/INCREMENTAL:NO'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_force_conformance_in_for_loop_scope(context, flag_name, node):
        """
        Set flag: ForceConformanceInForLoopScope

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Zc:forScope'},
            'false': {cl_flags: '/Zc:forScope-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_remove_unreferenced_code_data(context, flag_name, node):
        """
        Set flag: RemoveUnreferencedCodeData

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Zc:inline'},
            'false': {cl_flags: '/Zc:inline-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_openmp_support(context, flag_name, node):
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/openmp'},
            'false': {cl_flags: ''},
            default_value: {}
        }

        return flag_values

    def __set_use_debug_libraries(self, context, flag_name, md):
        """
        Set Use Debug Libraries flag: /MD

        """
        del flag_name
        if isinstance(md, NodeStub):
            md.text = 'false'

        setting = context.current_setting
        context.settings[setting]['use_debug_libs'] = 'true' in md.text
        message(
            context,
            'UseDebugLibraries : {}'.format(context.settings[setting]['use_debug_libs']),
            ''
        )
        # update default of RuntimeLibrary
        self.__set_default_flag(context, 'RuntimeLibrary')

    @staticmethod
    def __set_warning_level(context, flag_name, node):
        """
        Set Warning level for Windows: /W

        """
        del context, flag_name, node
        flag_values = {
            'TurnOffAllWarnings': {cl_flags: '/w'},
            'Level1': {cl_flags: '/W1'},
            'Level2': {cl_flags: '/W2'},
            'Level3': {cl_flags: '/W3'},
            'Level4': {cl_flags: '/W4'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_suppress_startup_banner(context, flag_name, node):
        del context, flag_name

        if isinstance(node, NodeStub):  # ignore default pass
            return None

        parent = node.getparent()
        if 'ClCompile' in parent.tag:
            return {
                'true': {cl_flags: '/nologo'},
                default_value: {}
            }

        return {
            'true': {ln_flags: '/NOLOGO'},
            default_value: {}
        }

    @staticmethod
    def __set_floating_point_exceptions(context, flag_name, node):
        """
        Set FloatingPointExceptions: /fp:except

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/fp:except'},
            'false': {cl_flags: '/fp:except-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_warning_as_errors(context, flag_name, node):
        """
        Set TreatWarningAsError: /WX

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/WX'},
            'false': {cl_flags: '/WX-'},
            default_value: {}
        }

        return flag_values

    def __set_disable_specific_warnings(self, context, flag_name, specific_warnings_node):
        """
        Set DisableSpecificWarnings: /wd*

        """
        if not specific_warnings_node.text:
            return

        flags = []
        for sw in specific_warnings_node.text.strip().split(";"):
            sw = sw.strip()
            if sw not in ('%(DisableSpecificWarnings)', ''):
                flag = '/wd{0}'.format(sw)
                flags.append(flag)
        self.flags[context.current_setting][flag_name][cl_flags] = flags
        message(context, 'DisableSpecificWarnings : {}'.format(';'.join(flags)), '')

    @staticmethod
    def __set_support_just_my_code(context, flag_name, node):
        """
        Handle SupportJustMyCode
        :param context:
        :param flag_name:
        :param node:
        :return:
        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/JMC'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_conformance_mode(context, flag_name, conformance_mode_node):
        """
        Handle ConformanceMode
        :param context:
        :param flag_name:
        :param conformance_mode_node:
        :return:
        """
        del context, flag_name, conformance_mode_node
        flag_values = {
            'true': {cl_flags: '/permissive-'},
            'false': {},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_language_standard(context, flag_name, language_standard_node):
        """
        Handle LanguageStandard
        :param context:
        :param flag_name:
        :param language_standard_node:
        :return:
        """
        del context, flag_name, language_standard_node
        flag_values = {
            'stdcpp14': {cl_flags: '/std:c++14'},
            'stdcpp17': {cl_flags: '/std:c++17'},
            'stdcpplatest': {cl_flags: '/std:c++latest'},
            default_value: {}
        }
        return flag_values

    def __set_compile_additional_options(self, context, flag_name, add_opts_node):
        """
        Set Additional options

        """
        add_opts = set_unix_slash(add_opts_node.text).split()
        ready_add_opts = []
        for opt in add_opts:
            if opt != '%(AdditionalOptions)':
                ready_add_opts.append(opt)
        self.flags[context.current_setting][flag_name][cl_flags] = ready_add_opts
        message(context, 'Compile Additional Options : {}'.format(ready_add_opts), '')

    def __set_link_additional_options(self, context, flag_name, add_opts_node):
        """
        Set Additional options

        """
        add_opts = set_unix_slash(add_opts_node.text).split()
        ready_add_opts = []
        for opt in add_opts:
            if opt != '%(AdditionalOptions)':
                ready_add_opts.append(opt)
        self.flags[context.current_setting][flag_name][ln_flags] = ready_add_opts
        message(context, 'Link Additional Options : {}'.format(ready_add_opts), '')

    @staticmethod
    def __set_basic_runtime_checks(context, flag_name, node):
        """
        Set Basic Runtime Checks flag: /RTC*

        """
        del context, flag_name, node
        flag_values = {
            'StackFrameRuntimeCheck': {cl_flags: '/RTCs'},
            'UninitializedLocalUsageCheck': {cl_flags: '/RTCu'},
            'EnableFastChecks': {cl_flags: '/RTC1'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_show_includes(context, flag_name, node):
        """
        Set ShowIncludes flag: /showIncludes*

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/showIncludes'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_compile_as_managed(context, flag_name, node):
        """
        Set CompileAsManaged flag: /clr*

        """
        del context, flag_name, node
        flag_values = {
            'Pure': {cl_flags: '/clr:pure'},
            'Safe': {cl_flags: '/clr:safe'},
            'true': {cl_flags: '/clr'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_enable_enhanced_instruction_set(context, flag_name, node):
        """
        Set EnableEnhancedInstructionSet flag: /arch:*

        """
        del context, flag_name, node
        flag_values = {
            'StreamingSIMDExtensions': {cl_flags: '/arch:SSE'},
            'StreamingSIMDExtensions2': {cl_flags: '/arch:SSE2'},
            'AdvancedVectorExtensions': {cl_flags: '/arch:AVX'},
            'AdvancedVectorExtensions2': {cl_flags: '/arch:AVX2'},
            'NoExtensions': {cl_flags: '/arch:IA32'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_omit_frame_pointers(context, flag_name, node):
        """
        Set OmitFramePointers flag: /Oy

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Oy'},
            'false': {cl_flags: '/Oy-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_calling_convention(context, flag_name, node):
        """
        Set CallingConvention flag: /G**

        """
        del context, flag_name, node
        flag_values = {
            'Cdecl': {cl_flags: '/Gd'},
            'FastCall': {cl_flags: '/Gr'},
            'StdCall': {cl_flags: '/Gz'},
            'VectorCall': {cl_flags: '/Gv'},
            default_value: {}
        }

        return flag_values

    def __set_runtime_library(self, context, flag_name, runtime_library_node):
        """
        Set RuntimeLibrary flag: /MDd

        """

        mdd = '/MDd'
        m_d = '/MD'
        mtd = '/MTd'
        m_t = '/MT'

        cl_flag_value = ''
        node_text = runtime_library_node.text
        if node_text:
            if node_text == 'MultiThreadedDebugDLL':
                cl_flag_value = mdd
            if node_text == 'MultiThreadedDLL':
                cl_flag_value = m_d
            if node_text == 'MultiThreaded':
                cl_flag_value = m_t
            if node_text == 'MultiThreadedDebug':
                cl_flag_value = mtd
        else:
            if context.file_contexts is not None:  # if not file context
                if isinstance(runtime_library_node, NodeStub):  # if default pass
                    setting = context.settings[context.current_setting]
                    if setting['use_debug_libs']:
                        cl_flag_value = '${DEFAULT_CXX_DEBUG_RUNTIME_LIBRARY}'
                    else:
                        cl_flag_value = '${DEFAULT_CXX_RUNTIME_LIBRARY}'
                    message(context, 'RuntimeLibrary : updating default...', '')

        if cl_flag_value:
            self.flags[context.current_setting][flag_name][cl_flags] = [cl_flag_value]
            message(context, 'RuntimeLibrary {}: {}'.format(node_text, cl_flag_value), '')

    @staticmethod
    def __set_string_pooling(context, flag_name, node):
        """
        Set StringPooling flag: /GF

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/GF'},
            'false': {cl_flags: '/GF-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_enable_fiber_safe_optimizations(context, flag_name, node):
        """
        Set EnableFiberSafeOptimizations flag: /GT

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/GT'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_minimal_rebuild(context, flag_name, node):
        """
        Set MinimalRebuild flag: /Gm

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Gm'},
            'false': {cl_flags: '/Gm-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_optimization(context, flag_name, node):
        """
        Set Optimization flag: /Od

        """
        del context, flag_name, node
        flag_values = {
            'Disabled': {cl_flags: '/Od'},
            'MinSpace': {cl_flags: '/O1'},
            'MaxSpeed': {cl_flags: '/O2'},
            'Full': {cl_flags: '/Ox'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_inline_function_expansion(context, flag_name, node):
        """
        Set Inline Function Expansion flag: /Ob

        """
        del context, flag_name, node
        flag_values = {
            'Disabled': {cl_flags: '/Ob0'},
            'OnlyExplicitInline': {cl_flags: '/Ob1'},
            'AnySuitable': {cl_flags: '/Ob2'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_intrinsic_functions(context, flag_name, node):
        """
        Set Intrinsic Functions flag: /Oi

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Oi'},
            'false': {cl_flags: '/Oi-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_sdl_check(context, flag_name, node):
        """
        Set SDLCheck flag: /sdl

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/sdl'},
            'false': {cl_flags: '/sdl-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_runtime_type_info(context, flag_name, node):
        """
        Set RuntimeTypeInfo flag: /GR

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/GR'},
            'false': {cl_flags: '/GR-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_function_level_linking(context, flag_name, node):
        """
        Set FunctionLevelLinking flag: /Gy

        """
        del context, flag_name, node
        flag_values = {
            'true': {cl_flags: '/Gy'},
            'false': {cl_flags: '/Gy-'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_debug_information_format(context, flag_name, node):
        """
        Set DebugInformationFormat flag: /Zi

        """

        if context.file_contexts is None and node.text == '':  # if file context ignore default
            return {}

        del context, flag_name, node
        flag_values = {
            'ProgramDatabase': {cl_flags: '/Zi'},
            'EditAndContinue': {cl_flags: '/ZI'},
            default_value: {cl_flags: '${DEFAULT_CXX_DEBUG_INFORMATION_FORMAT}'}
        }

        return flag_values

    @staticmethod
    def __set_assembler_listing_location(context, flag_name, node):
        """
        Set AssemblerListingLocation flag: /Fa

        """
        del flag_name, node
        flag_values = {
            default_value: {}
        }

        # /Fa breaks non Visual Studio CMake generators.
        # In visual studio MSBuild target MakeDirsForCl creates dir before compiling but
        # other generators (Ninja, NMake Makefiles) don't.
        # Then missing target asm directory error occurs.
        # if node.text:
        #     flag_values.update(
        #         {
        #             node.text: {
        #                 cl_flags: '/Fa{}'.format(cleaning_output(context, node.text))
        #             }
        #         }
        #     )

        message(
            context,
            '/Fa option is ignored. Too hard to handle for different CMake generators.',
            'warn4'
        )

        return flag_values

    @staticmethod
    def __set_assembler_output(context, flag_name, node):
        """
        Set AssemblerOutput flag: /FA

        """
        del context, flag_name, node
        flag_values = {
            'AssemblyCode': {cl_flags: '/FA'},
            'AssemblyAndMachineCode': {cl_flags: '/FAc'},
            'AssemblyAndSourceCode': {cl_flags: '/FAs'},
            'All': {cl_flags: '/FAcs'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_object_file_name(context, flag_name, node):
        """
        Set ObjectFileName flag: /Fo

        """
        del context, flag_name, node
        flag_values = {
            default_value: {}
        }

        # /Fo is hardcoded inside CMake. No way to control this.
        # if node.text:
        #     flag_values.update(
        #         {
        #             node.text: {
        #                 cl_flags: '/Fo{}'.format(cleaning_output(context, node.text))
        #             }
        #         }
        #     )

        return flag_values

    @staticmethod
    def __set_favour_size_or_speed(context, flag_name, node):
        """
        Set FavorSizeOrSpeed: /Os, /Ot

        """
        del context, flag_name, node
        flag_values = {
            'Size': {cl_flags: '/Os'},
            'Speed': {cl_flags: '/Ot'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_compile_as(context, flag_name, node):
        """
        Set Compile As flag: /TP, TC

        """
        del context, flag_name, node
        flag_values = {
            'CompileAsCpp': {cl_flags: '/TP'},
            'CompileAsC': {cl_flags: '/TC'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_image_has_safe_exception_handlers(context, flag_name, node):
        """
        Set ImageHasSafeExceptionHandlers flag: /SAFESEH

        """
        del context, flag_name, node
        flag_values = {
            'false': {ln_flags: '/SAFESEH:NO'},
            'true': {ln_flags: '/SAFESEH'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_sub_system(context, flag_name, node):
        """
        Set SubSystem flag: /SUBSYSTEM

        """
        del context, flag_name, node
        flag_values = {
            'Console': {ln_flags: '/SUBSYSTEM:CONSOLE'},
            'Windows': {ln_flags: '/SUBSYSTEM:WINDOWS'},
            'Native': {ln_flags: '/SUBSYSTEM:NATIVE'},
            'EFI Application': {ln_flags: '/SUBSYSTEM:EFI_APPLICATION'},
            'EFI Boot Service Driver': {ln_flags: '/SUBSYSTEM:EFI_BOOT_SERVICE_DRIVER'},
            'EFI ROM': {ln_flags: '/SUBSYSTEM:EFI_ROM'},
            'EFI Runtime': {ln_flags: '/SUBSYSTEM:EFI_RUNTIME_DRIVER'},
            'POSIX': {ln_flags: '/SUBSYSTEM:POSIX'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_optimize_references(context, flag_name, node):
        """
        Set OptimizeReferences flag: /OPT:REF

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/OPT:REF'},
            'false': {ln_flags: '/OPT:NOREF'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_enable_comdat_folding(context, flag_name, node):
        """
        Set EnableCOMDATFolding flag: /OPT:ICF

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/OPT:ICF'},
            'false': {ln_flags: '/OPT:NOICF'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_generate_debug_information(context, flag_name, node):
        """
        Set GenerateDebugInformation flag: /DEBUG

        """
        del context, flag_name, node
        flag_values = {
            'DebugFull': {ln_flags: '/DEBUG:FULL'},
            'DebugFastLink': {ln_flags: '/DEBUG:FASTLINK'},
            'true': {ln_flags: '/DEBUG'},
            'false': {},
            default_value: {ln_flags: '/DEBUG:FULL'}
        }

        return flag_values

    @staticmethod
    def __set_target_machine(context, flag_name, node):
        """
        Set TargetMachine flag: /MACHINE

        """
        del context, flag_name, node
        flag_values = {
            'MachineARM': {ln_flags: '/MACHINE:ARM'},
            'MachineARM64': {ln_flags: '/MACHINE:ARM64'},
            'MachineEBC': {ln_flags: '/MACHINE:EBC'},
            'MachineIA64': {ln_flags: '/MACHINE:IA64'},
            'MachineMIPS': {ln_flags: '/MACHINE:MIPS'},
            'MachineMIPS16': {ln_flags: '/MACHINE:MIPS16'},
            'MachineMIPSFPU': {ln_flags: '/MACHINE:MIPSFPU'},
            'MachineMIPSFPU16': {ln_flags: '/MACHINE:MIPSFPU16'},
            'MachineTHUMB': {ln_flags: '/MACHINE:THUMB'},
            'MachineX64': {ln_flags: '/MACHINE:X64'},
            'MachineX86': {ln_flags: '/MACHINE:X86'},
            'false': {},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_fixed_base_address(context, flag_name, node):
        """
        Set FixedBaseAddress flag: /FIXED

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/FIXED'},
            'false': {ln_flags: '/FIXED:NO'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_stack_reserve_size(context, flag_name, node):
        """
        Set StackReserveSize flag: /STACK

        """
        del context, flag_name
        flag_values = {
            default_value: {}
        }

        if node.text:
            flag_values.update(
                {
                    node.text: {
                        ln_flags: '/STACK:{}'.format(node.text)
                    }
                }
            )

        return flag_values

    @staticmethod
    def __set_profile(context, flag_name, node):
        """
        Set DataExecutionPrevention flag: /NXCOMPAT

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/PROFILE'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_data_execution_prevention(context, flag_name, node):
        """
        Set DataExecutionPrevention flag: /NXCOMPAT

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/NXCOMPAT'},
            'false': {ln_flags: '/NXCOMPAT:NO'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_randomized_base_address(context, flag_name, node):
        """
        Set RandomizedBaseAddress flag: /DYNAMICBASE

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/DYNAMICBASE'},
            'false': {ln_flags: '/DYNAMICBASE:NO'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_ignore_embedded_idl(context, flag_name, node):
        """
        Set IgnoreEmbeddedIDL flag: /IGNOREIDL

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/IGNOREIDL'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_assembly_debug(context, flag_name, node):
        """
        Set AssemblyDebug flag: /ASSEMBLYDEBUG

        """
        del context, flag_name, node
        flag_values = {
            'true': {ln_flags: '/ASSEMBLYDEBUG'},
            'false': {ln_flags: '/ASSEMBLYDEBUG:DISABLE'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_floating_point_model(context, flag_name, node):
        """
        Set FloatingPointModel flag: /fp

        """
        del context, flag_name, node
        flag_values = {
            'Precise': {cl_flags: '/fp:precise'},
            'Strict': {cl_flags: '/fp:strict'},
            'Fast': {cl_flags: '/fp:fast'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_struct_member_alignment(context, flag_name, node):
        """
        Set StructMemberAlignment flag: /Zp

        """
        del context, flag_name, node
        flag_values = {
            '1Byte': {cl_flags: '/Zp1'},
            '2Bytes': {cl_flags: '/Zp2'},
            '4Bytes': {cl_flags: '/Zp4'},
            '8Bytes': {cl_flags: '/Zp8'},
            '16Bytes': {cl_flags: '/Zp16'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_exception_handling(context, flag_name, node):
        """
        Set ExceptionHandling flag: /EHsc

        """
        if context.file_contexts is None and node.text == '':  # if file context ignore default
            return {}

        del context, flag_name, node
        flag_values = {
            'false': {},
            'true': {cl_flags: '/EHsc'},
            'Async': {cl_flags: '/EHa'},
            default_value: {cl_flags: '${DEFAULT_CXX_EXCEPTION_HANDLING}'}
        }

        return flag_values

    @staticmethod
    def __set_buffer_security_check(context, flag_name, node):
        """
        Set BufferSecurityCheck flag: /GS

        """
        del context, flag_name, node
        flag_values = {
            'false': {cl_flags: '/GS-'},
            'true': {cl_flags: '/GS'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_control_flow_guard(context, flag_name, node):
        """
        Set ControlFlowGuard flag: /guard:cf

        """
        del context, flag_name, node
        flag_values = {
            'Guard': {cl_flags: '/guard:cf'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_diagnostics_format(context, flag_name, node):
        """
        Set DiagnosticsFormat flag : /diagnostics

        """
        del context, flag_name, node
        flag_values = {
            'Classic': {cl_flags: '/diagnostics:classic'},
            'Column': {cl_flags: '/diagnostics:column'},
            'Caret': {cl_flags: '/diagnostics:caret'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_disable_language_extensions(context, flag_name, node):
        """
        Set DisableLanguageExtensions /Za

        """
        del context, flag_name, node
        flag_values = {
            'false': {},
            'true': {cl_flags: '/Za'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __set_treatwchar_t_as_built_in_type(context, flag_name, node):
        """
        Set TreatWChar_tAsBuiltInType /Zc:wchar_t

        """
        del context, flag_name, node
        flag_values = {
            'false': {cl_flags: '/Zc:wchar_t-'},
            'true': {cl_flags: '/Zc:wchar_t'},
            default_value: {}
        }

        return flag_values

    @staticmethod
    def __setting_has_pch(context, setting):
        """
        Return if there is precompiled header or not for given setting

        :param context: converter Context
        :type context: Context
        :param setting: related setting (Release|x64, Debug|Win32,...)
        :type setting: str
        :return: if use PCH or not
        :rtype: bool
        """

        has_pch = context.settings[setting]['PrecompiledHeader']

        return 'Use' in has_pch

    @staticmethod
    def write_precompiled_headers(context, setting, cmake_file):
        """
        Write precompiled headers, if needed, on given CMake file

        :param context: converter Context
        :type context: Context
        :param setting: related setting (Release|x64, Debug|Win32,...)
        :type setting: str
        :param cmake_file: CMakeLIsts.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        pch_header = context.settings[setting]['PrecompiledHeaderFile']
        pch_source = context.settings[setting]['PrecompiledSourceFile']
        working_path = os.path.dirname(context.vcxproj_path)
        cmake_file.write(
            'add_precompiled_header(${{PROJECT_NAME}} "{0}" "{1}")\n\n'.format(
                os.path.basename(pch_header),
                normalize_path(context, working_path, pch_source, False)
            )
        )

    def write_use_pch_function(self, context, cmake_file):
        need_pch = False
        any_setting = None
        for setting in context.settings:
            if self.__setting_has_pch(context, setting):
                need_pch = True
                any_setting = setting
                break

        if need_pch:
            self.write_precompiled_headers(context, any_setting, cmake_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    ProjectFiles
    =============
     Manages the recovery of project files
"""

import os
import copy

from cmake_converter.utils import take_name_from_list_case_ignore, normalize_path
from cmake_converter.utils import message, write_comment, make_cmake_literal


class ProjectFiles:
    """
        Class who collect and store project files
    """

    def __init__(self):
        self.languages = []
        self.file_lists = {}
        self.file_lists_for_include_paths = {}

    def include_directive_case_check(self, context, file_path_name, file_lists_for_include_paths):
        pass

    @staticmethod
    def __create_file_context(context):
        file_context = copy.copy(context)
        file_context.settings = {}
        file_context.flags = copy.copy(context.flags)
        file_context.flags.__init__()
        file_context.file_contexts = None
        file_context.warnings_count = 0
        for setting in context.settings:       # copy settings
            file_context.current_setting = setting
            file_context.utils.init_context_current_setting(file_context)
        context.current_setting = (None, None)
        return file_context

    def __add_file_into_container(self, context, **kwargs):

        files_container = kwargs['files_container']
        file_path = kwargs['file_path']
        file_name = kwargs['file_name']
        source_group = kwargs['source_group']

        real_name = take_name_from_list_case_ignore(context, self.file_lists[file_path],
                                                    file_name)
        if real_name:
            name_to_add = real_name
        else:
            name_to_add = file_name
            message(context, 'Adding absent {} file into project files'
                    .format(file_name), 'warn')

        files_container[file_path].append(name_to_add)
        if file_path:
            file_path = file_path + '/'
        file_path_name = file_path + name_to_add
        working_path = os.path.dirname(context.vcxproj_path)
        file_path_name = normalize_path(context, working_path,
                                        file_path_name,
                                        False)
        if source_group not in context.source_groups:
            context.source_groups[source_group] = []
        context.source_groups[source_group].append(file_path_name)
        context.source_groups[source_group].sort(key=str.lower)
        if real_name:
            self.include_directive_case_check(context,
                                              file_path_name,
                                              self.file_lists_for_include_paths)
        context.file_contexts[file_path_name] = self.__create_file_context(context)
        return context.file_contexts[file_path_name]

    def add_file_from_node(self, context, **kwargs):

        files_container = kwargs['files_container']
        file_node = kwargs['file_node']
        file_node_attr = kwargs['file_node_attr']
        source_group = kwargs['source_group']

        if file_node.get(file_node_attr) is not None:
            node_text = str(file_node.get(file_node_attr))
            node_text = '/'.join(node_text.split('\\'))
            if not node_text.rpartition('.')[-1] in self.languages:
                self.languages.append(node_text.rpartition('.')[-1])
            file_path, file_name = os.path.split(node_text)
            if file_path not in self.file_lists:
                vcxproj_dir = os.path.dirname(context.vcxproj_path)
                self.file_lists[file_path] = []
                if os.path.exists(os.path.join(vcxproj_dir, file_path)):
                    self.file_lists[file_path] = os.listdir(os.path.join(vcxproj_dir, file_path))
            if file_path not in files_container:
                files_container[file_path] = []
            if file_name not in files_container[file_path]:
                return self.__add_file_into_container(
                    context,
                    files_container=files_container,
                    file_path=file_path,
                    file_name=file_name,
                    source_group=source_group
                )
        return ''

    def init_file_lists_for_include_paths(self, context):
        """
        For include directive case ad path checking. Works only with vfproj.
        :param context:
        :return:
        """
        vcxproj_dir = os.path.dirname(context.vcxproj_path)
        for setting in context.settings:
            for include_path in context.settings[setting]['inc_dirs_list']:
                if include_path not in self.file_lists_for_include_paths:
                    abs_include_path = os.path.join(vcxproj_dir, include_path)
                    if os.path.exists(abs_include_path):
                        self.file_lists_for_include_paths[include_path]\
                            = os.listdir(abs_include_path)

    def apply_files_to_context(self, context):
        has_headers = True if context.headers else False
        context.has_headers = has_headers
        context.has_only_headers = True if has_headers and not context.sources else False
        message(context, "Source files extensions found: {0}".format(self.languages), 'INFO')

    def find_cmake_project_languages(self, context):
        """
        Add CMake Project

        """

        cpp_extensions = ['cc', 'cp', 'cxx', 'cpp', 'CPP', 'c++', 'C']

        available_language = {'c': 'C'}
        available_language.update(dict.fromkeys(cpp_extensions, 'CXX'))

        fortran_extensions = ['F90', 'F', 'f90', 'f', 'fi', 'FI']
        available_language.update(dict.fromkeys(fortran_extensions, 'Fortran'))

        project_languages_set = set()
        for l in self.languages:
            if l in available_language:
                project_languages_set.add(available_language[l])

        project_languages = list(project_languages_set)
        project_languages.sort()
        for project_language in project_languages:
            context.solution_languages.add(project_language)
        context.project_languages = project_languages

    @staticmethod
    def write_cmake_project(context, cmake_file):
        """
        Write cmake project for given CMake file

        :param context: Converter context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        lang = ''
        if context.project_languages:
            lang = ' ' + ' '.join(context.project_languages)
        if context.project_name == '':
            message(
                context,
                'No PROJECT NAME found or define. '
                'Check it!!',
                'error'
            )
        cmake_file.write(
            'project({0}{1})\n\n'.format(make_cmake_literal(context, context.project_name), lang)
        )

    @staticmethod
    def get_source_group_var(context, source_group_name):
        if not source_group_name:
            return 'no_group_source_files'

        source_group_name = source_group_name.replace(' ', '_')
        return make_cmake_literal(context, source_group_name.replace('\\\\', '__'))

    def write_source_groups(self, context, cmake_file):
        write_comment(cmake_file, 'Source groups')

        source_group_list = sorted(context.source_groups)
        for source_group in source_group_list:
            source_group_var = self.get_source_group_var(context, source_group)
            cmake_file.write('set({}\n'.format(source_group_var))
            for src_file in context.source_groups[source_group]:
                cmake_file.write('    {0}\n'.format(src_file))

            cmake_file.write(')\n')
            cmake_file.write(
                'source_group("{}" FILES ${{{}}})\n\n'.format(source_group, source_group_var)
            )

        cmake_file.write('set(ALL_FILES\n')
        for source_group in source_group_list:
            cmake_file.write(
                '    ${{{}}}\n'.format(context.files.get_source_group_var(context, source_group))
            )
        cmake_file.write(')\n\n')

    @staticmethod
    def add_additional_code(context, file_to_add, cmake_file):
        """
        Add additional file with CMake code inside

        :param context: the context of converter
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        :param file_to_add: the file who contains CMake code
        :type file_to_add: str
        """

        if file_to_add != '':
            try:
                fc = open(file_to_add)
                write_comment(cmake_file, 'Provides from external file.')
                for line in fc:
                    cmake_file.write(line)
                fc.close()
                cmake_file.write('\n')
                message(context, 'File of Code is added = ' + file_to_add, 'warn')
            except OSError as e:
                message(context, str(e), 'error')
                message(
                    context,
                    'Wrong data file ! Code was not added, please verify file name or path !',
                    'error'
                )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from cmake_converter.context import Context
from cmake_converter.data_files import get_vcxproj_data, get_cmake_lists


@unittest.skip("TestProjectVariables tests skipping")
class TestProjectVariables(unittest.TestCase):
    """
        This file test methods of ProjectVariables class.
    """

    cur_dir = os.path.dirname(os.path.realpath(__file__))
    context = Context()
    vcxproj_data_test = get_vcxproj_data(context, '%s/datatest/foo.vcxproj' % cur_dir)
    cmake_lists_test = get_cmake_lists(context, cur_dir)

    data_test = {
        'cmake': cmake_lists_test,
        'cmakeoutput': None,
        'vcxproj': vcxproj_data_test,
        'include': None,
        'additional': None,
    }

    def test_init_project_variables(self):
        """Initialize Project Variables"""

        under_test = VCXProjectVariables(self.data_test)

        self.assertTrue(under_test.tree)
        self.assertTrue(under_test.ns)
        self.assertTrue(under_test.cmake)
        self.assertFalse(under_test.output)
        self.assertIsNotNone(under_test.cmake_outputs)

    def test_add_project_variables(self):
        """Add Project Variables"""

        self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = VCXProjectVariables(self.data_test)

        under_test.add_project_variables()

        self.data_test['cmake'].close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_test = cmakelists_test.read()

        self.assertTrue('set(PROJECT_NAME core)' in content_test)

        cmakelists_test.close()

    def test_add_outputs_variables(self):
        """Add Outputs Variables"""

        # TODO If NO output is given
        # self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = VCXProjectVariables(self.data_test)
        #
        # under_test.add_project_variables()
        # under_test.add_outputs_variables()
        #
        # self.data_test['cmake'].close()
        #
        # cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        # content_test = cmakelists_test.read()
        #
        # self.assertTrue('OUTPUT_DEBUG ../../../build/vc2017_x64d/bin/', content_test)
        # self.assertTrue('OUTPUT_REL ../../../build/vc2017_x64/bin/' in content_test)
        #
        # cmakelists_test.close()

        # If output is given
        under_test.output = '../output_binaries'
        under_test.cmake = get_cmake_lists(context, self.cur_dir)
        under_test.add_outputs_variables()

        under_test.cmake.close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_test = cmakelists_test.read()

        self.assertTrue('OUTPUT_DEBUG ../output_binaries/${CMAKE_BUILD_TYPE}', content_test)
        self.assertTrue('OUTPUT_REL ../output_binaries/${CMAKE_BUILD_TYPE}' in content_test)

        cmakelists_test.close()

    def test_add_cmake_project(self):
        """Add CMake Project"""

        self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = VCXProjectVariables(self.data_test)

        # Case CXX languages
        under_test.add_cmake_project(['cpp'])

        self.data_test['cmake'].close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir, 'r')
        content_test = cmakelists_test.read()

        self.assertTrue('project(${PROJECT_NAME} CXX)' in content_test)

        cmakelists_test.close()

        # Case C languages
        under_test.cmake = get_cmake_lists(context, self.cur_dir)
        under_test.add_cmake_project(['c'])

        under_test.cmake.close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir, 'r')
        content_test = cmakelists_test.read()

        self.assertTrue('project(${PROJECT_NAME} C)' in content_test)

        cmakelists_test.close()

    def test_add_artefact_target_outputs(self):
        """Add Artefact Target Outputs"""

        self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = VCXProjectVariables(self.data_test)

        under_test.add_cmake_output_directories()

        self.data_test['cmake'].close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir, 'r')
        content_test = cmakelists_test.read()

        self.assertTrue(
            'set(CMAKE_LIBRARY_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_DEBUG}")' in
            content_test
        )
        self.assertTrue(
            'set(CMAKE_ARCHIVE_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_DEBUG}")' in
            content_test
        )
        self.assertTrue(
            'set(CMAKE_EXECUTABLE_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_DEBUG}' in
            content_test
        )

        self.assertTrue(
            'set(CMAKE_LIBRARY_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_RELEASE}")' in
            content_test
        )
        self.assertTrue(
            'set(CMAKE_ARCHIVE_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_RELEASE}")' in
            content_test
        )
        self.assertTrue(
            'set(CMAKE_EXECUTABLE_OUTPUT_DIRECTORY "${PROJECT_DIR}/${OUTPUT_RELEASE}' in
            content_test
        )

        cmakelists_test.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os

from cmake_converter.dependencies import Dependencies
from cmake_converter.utils import message, prepare_build_event_cmd_line_for_cmake,\
    check_for_relative_in_path, cleaning_output


class VFDependencies(Dependencies):

    @staticmethod
    def set_target_references(context):
        if context.sln_deps:
            context.target_references = context.target_references + context.sln_deps
            message(context, 'References : {}'.format(context.target_references), '')

    @staticmethod
    def set_target_additional_dependencies(context, flag_name, ad_libs, node):
        del flag_name, node

        if ad_libs:
            add_libs = []
            for d in ad_libs.split(';'):
                if d != '%(AdditionalDependencies)':
                    if os.path.splitext(d)[1] == '.lib':
                        add_libs.append(d.replace('.lib', ''))
            context.add_lib_deps = True
            message(context, 'Additional Dependencies = {0}'.format(add_libs), '')
            context.settings[context.current_setting]['add_lib_deps'] = add_libs

    @staticmethod
    def set_target_additional_library_directories(context, flag_name,
                                                  additional_library_directories, node):
        """
        Find and set additional library directories in context

        """
        del flag_name, node

        if additional_library_directories:
            list_depends = additional_library_directories.replace(
                '%(AdditionalLibraryDirectories)', ''
            )
            if list_depends != '':
                add_lib_dirs = []
                for d in list_depends.split(';'):
                    d = d.strip()
                    if d != '':
                        add_lib_dirs.append(
                            check_for_relative_in_path(
                                context,
                                cleaning_output(context, d)
                            )
                        )
                message(context,
                        'Additional Library Directories = {0}'.format(add_lib_dirs), '')
                context.settings[context.current_setting]['target_link_dirs'] = add_lib_dirs

    @staticmethod
    def __set_custom_build_events_of_files(context, name, command_value, node):
        del name, command_value

        if 'file_custom_build_events' not in context.settings[context.current_setting]:
            context.settings[context.current_setting]['file_custom_build_events'] = {}
        custom_event_data = {
            'command_line': prepare_build_event_cmd_line_for_cmake(
                context,
                node.attrib['CommandLine']
            ),
            'description': node.attrib['Description'],
            'outputs': node.attrib['Outputs'],
        }
        context.settings[context.current_setting]['file_custom_build_events'] = custom_event_data

    @staticmethod
    def __set_target_build_events(context, value_name, event_type, command_value):
        context.settings[context.current_setting][value_name] = []
        for build_event in command_value.split('\n'):
            build_event = build_event.strip()
            if build_event:
                cmake_build_event = prepare_build_event_cmd_line_for_cmake(
                    context,
                    build_event
                )
                context.settings[context.current_setting][value_name] \
                    .append(cmake_build_event)
                message(context, '{0} event for {1}: {2}'
                        .format(event_type, context.current_setting, cmake_build_event), 'info')

    @staticmethod
    def __is_excluded_from_build(node):
        if 'ExcludedFromBuild' in node.attrib:
            return node.attrib['ExcludedFromBuild'] == 'true'
        return False

    def set_target_pre_build_events(self, context, name, command_value, node):
        del name

        if self.__is_excluded_from_build(node):
            return
        self.__set_target_build_events(
            context,
            'pre_build_events',
            'Pre build',
            command_value
        )

    def set_target_pre_link_events(self, context, name, command_value, node):
        del name

        if self.__is_excluded_from_build(node):
            return
        self.__set_target_build_events(
            context,
            'pre_link_events',
            'Pre link',
            command_value
        )

    def set_target_post_build_events(self, context, name, command_value, node):
        del name

        if self.__is_excluded_from_build(node):
            return
        self.__set_target_build_events(
            context,
            'post_build_events',
            'Post build',
            command_value
        )

    # TODO: implement
    def set_custom_build_step(self, context, name, command_value, node):
        if self.__is_excluded_from_build(node):
            return
        self.__set_custom_build_events_of_files(context, name, command_value, node)
    #     self.__find_target_build_events(
    #         context,
    #         'VFCustomBuildTool',
    #         'custom_build_step',
    #         'Custom build'
    #     )

    def write_target_property_sheets(self, context, cmake_file):
        """
        Write target property sheets of current context

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """
        cmake_file.write(
            'use_props(${PROJECT_NAME} "${CMAKE_CONFIGURATION_TYPES}" "${DEFAULT_Fortran_PROPS}")\n'
        )
        super(VFDependencies, self).write_target_property_sheets(context, cmake_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest
import lxml
import _io

from cmake_converter.data_files import get_vcxproj_data, get_cmake_lists
from cmake_converter.data_files import get_propertygroup, get_definitiongroup
from cmake_converter.context import Context


class TestDataFiles(unittest.TestCase):
    """
        This file test 'data_files' functions
    """

    cur_dir = os.path.dirname(os.path.realpath(__file__))
    vs_project = '%s/datatest/foo.vcxproj' % cur_dir

    def test_get_vcxproj_data(self):
        """Get VS Project Data"""

        context = Context()
        under_test = get_vcxproj_data(context, self.vs_project)

        self.assertTrue('ns' in under_test)
        self.assertEqual(
            {'ns': 'http://schemas.microsoft.com/developer/msbuild/2003'},
            under_test['ns']
        )
        self.assertTrue('tree' in under_test)
        self.assertIsInstance(under_test['tree'], lxml.etree._ElementTree)

    def test_get_propertygroup(self):
        """Get Property Group"""

        under_test = get_propertygroup(('Release', 'x64'), 'and @Label="Configuration"')

        self.assertTrue('PropertyGroup' in under_test)
        self.assertTrue('Release|x64' in under_test)

    def test_get_definitiongroup(self):
        """Get Definition Group"""

        under_test = get_definitiongroup(('Release', 'Win32'))

        self.assertTrue('ItemDefinitionGroup' in under_test)
        self.assertTrue('Release|Win32' in under_test)

    def test_get_cmakelists(self):
        """Get CMakeLists.txt"""

        context = Context()
        under_test = get_cmake_lists(context, './')

        self.assertTrue(under_test)
        self.assertIsInstance(under_test, _io.TextIOWrapper)
        under_test.close()
